// src/cairo_elements.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef CAIRO_ELEMENTS_HH
#define CAIRO_ELEMENTS_HH 1

#include "config.h"

#include "backend/thread.hh"
#include "backend/process.hh"

#include <cairo/cairo.h>

namespace sgpem
{

  struct Point
  {
    float x;
    float y;
  };

  struct Rectangle
  {
    float x0;
    float y0;
    float w;
    float h;
  };

  struct Color
  {
    float r;
    float g;
    float b;
  };


  class CairoElements
  {
  public:
    CairoElements(cairo_t* const ctx);
    
    void draw_3dsphere(const Point& center, float radius, const Color& cl);
    void draw_3dcube(const Rectangle& area, const Color& cl);
    void draw_container(const Rectangle& area);
    void draw_expandable(const Rectangle& area, bool expanded = false);

    Rectangle draw_thread_status(const Point& top_left_crn, 
                                 const Thread& thread_info);
    Rectangle draw_process_status(const Point& top_left_crn,
                                  const Process& process_info);

  private:
    cairo_t*  _ctx;
  };

} //~ namespace sgpem


#endif //~ CAIRO_ELEMENTS_HH
